def f():
    a = []
    for e in range(10):
        a.append(e)
    b = set()
    for e in range(10):
        b.add(e)
    payload = {}
    for field in model.__searchable__:
        for i in range(1):
            payload[field] = getattr(model, field)
    if i > n1 and i <= n1 + n2 and 0 == rc:
        print("test chain compare")
    if self.pendingRescans == []:
        print("test truth value test")
    e = throttling_mod_func(d, e)
    f = d[0]
    d[0] = d[e]
    d[e] = f
    for i in matches:
        i[0]
        i[1]
    for i in matches:
        string = string.replace(i[0], i[1] + '_')
        symbols.append(i[1])
    finishedForLoop = True
    for x in range(2, n):
        if n % x == 0:
            finishedForLoop = False
            break
    if finishedForLoop:
        pass
    dicts = load_crowdhuman_json(sys.argv[1], sys.argv[2], sys.argv[3])



from typing import NoReturn
import sys,os,argparse
abs_path=os.path.abspath(os.path.dirname(__file__))

# abs_path=os.path.abspath(os.path.dirname(__file__))
# # print("abs_path: ",abs_path)
# pack_path="/".join(abs_path.split("/")[:-1])
# # print(pack_path)
# sys.path.append(pack_path)

from RefactoringIdioms import main
__all__ = ["run_main"]

def run_main():
    abs_path=os.getcwd()
    # print("hello: ",abs_path)
    parser = argparse.ArgumentParser(description='RefactoringIdioms')
    parser.add_argument('--envname', type=str,
                        help='envname', default="CartPole-v0")
    # parser.add_argument('--filepath', type=str,
    #                     help='filepath', default=abs_path + "/file.py")
    # parser.add_argument('--filepath', type=str,
    #                     help='filepath', default=abs_path + "/")
    parser.add_argument('--filepath', type=str,
                        help='filepath', default= "./")

    parser.add_argument('--idiom', type=str,
                        help='idiom', default="All")
    parser.add_argument('--outputdir', type=str,
                        help='outputdir', default="./RefactoringIdiomsOutputdir/")
    parser.add_argument('--flag_merge_allcodepair', type=str,
                        help='flag_merge_allcodepair', default=1)
    parser.add_argument('--output_codepair', type=str,
                        help='output_codepair', default="result.json")
    parser.add_argument('--flag_console_output', type=str,
                        help='flag_console_output', default=1)
    args = parser.parse_args()


    try:
        main.main(args)
    except KeyboardInterrupt:
        sys.exit(1)
# run_main()


def test_stop_training_csv(self):
    # Test that using the CSVLogger callback with the TerminateOnNaN callback
    # does not result in invalid CSVs.
    np.random.seed(1337)
    tmpdir = self.get_temp_dir()
    self.addCleanup(shutil.rmtree, tmpdir, ignore_errors=True)

    with self.cached_session():
        fp = os.path.join(tmpdir, 'test.csv')
        (x_train, y_train), (x_test, y_test) = testing_utils.get_test_data(
            train_samples=TRAIN_SAMPLES,
            test_samples=TEST_SAMPLES,
            input_shape=(INPUT_DIM,),
            num_classes=NUM_CLASSES)

        y_test = np_utils.to_categorical(y_test)
        y_train = np_utils.to_categorical(y_train)
        cbks = [keras.callbacks.TerminateOnNaN(), keras.callbacks.CSVLogger(fp)]
        model = keras.models.Sequential()
        for _ in range(5):
            model.add(keras.layers.Dense(2, input_dim=INPUT_DIM, activation='relu'))
        model.add(keras.layers.Dense(NUM_CLASSES, activation='linear'))
        model.compile(loss='mean_squared_error',
                      optimizer='rmsprop')

        history = model.fit_generator(data_generator(),
                                      len(x_train) // BATCH_SIZE,
                                      validation_data=(x_test, y_test),
                                      callbacks=cbks,
                                      epochs=20)
        loss = history.history['loss']
        assert len(loss) > 1
        assert loss[-1] == np.inf or np.isnan(loss[-1])
    values = []
    with open(fp) as f:
        # On Windows, due to \r\n line ends, we may end up reading empty lines
        # after each line. Skip empty lines.
        for x in csv.reader(f):
            if x:
                values.append(x)
    assert 'nan' in values[-1], 'The last epoch was not logged.'



import sys,ast,os
import tokenize
# sys.path.append("/mnt/zejun/smp/code1/")
abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
sys.path.append("/".join(abs_path.split("/")[:-2]))
# print("abs_path_2: ","/".join(abs_path.split("/")[:-2]))
import complicated_code_util
import util
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
# from code1.extract_simp_cmpl_data import ast_util
from tokenize import tokenize
from io import BytesIO

# dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")

'''
get comprehension node
Block=> Assign;...; For
For=>”for” ( For| If|If-else|body)
If=>”if” (body| For| If| If-else)
If-else=>”if” body “else” (body| If-else)
body=> (Assign|Call)*;Append*
并且里面只有一个写变量
for

    b.append()
    
    a.append(b)

'''
# const_empty_list=["[]"]#["dict()","{}"]#["()"]
#
# const_func_name="add"

def get_func_name(one_body):
    pre_name,call_name=None,None
    if isinstance(one_body, ast.Expr) and isinstance(one_body.value, ast.Call):
        # print("come here")
        call_front = one_body.value.func
        if isinstance(call_front, ast.Name):
            call_name = ast.unparse(call_front)
        elif isinstance(call_front, ast.Attribute):
            call_name = call_front.attr
            pre_name=ast.unparse(call_front.value)
        else:
            call_name = ast.unparse(call_front)
    return pre_name,call_name


def whether_fun_is_append(one_body,assign_block_list,const_func_name="append"):
    #print("one_body: ",ast.unparse(one_body),one_body.__dict__)
    if isinstance(one_body, ast.Expr) and isinstance(one_body.value, ast.Call):
        # print("come here")
        call_front = one_body.value.func

        # print("call dict: ", call_front.__dict__)

        if isinstance(call_front, ast.Name):
            call_name = ast.unparse(call_front)
        elif isinstance(call_front, ast.Attribute):
            call_name = call_front.attr
        else:
            call_name = ast.unparse(call_front)
        #print("the func name: ", call_name)
        if call_name ==const_func_name:
            assign_block_list.append([one_body])
            return True
    elif isinstance(one_body,ast.Call):
        call_front = one_body.func

        # print("call dict: ", call_front.__dict__)
        if isinstance(call_front, ast.Name):
            call_name = ast.unparse(call_front)
        elif isinstance(call_front, ast.Attribute):
            call_name = call_front.attr
        else:
            call_name = ast.unparse(call_front)
        # print("the func name: ", call_name)
        if call_name == const_func_name:
            assign_block_list.append([one_body])
            return True
    return False

def else_traverse(if_body,assign_block_list,const_func_name):
    else_body_list = if_body.orelse
    if len(else_body_list) > 1:
        return False
    else:
        else_body_one = else_body_list[0]
        if isinstance(else_body_one, ast.If):
            or_else = else_body_one.orelse
            if not or_else:
                return False
            else:
                return if_else_traverse(else_body_one,assign_block_list,const_func_name)
        return whether_fun_is_append(else_body_one,assign_block_list,const_func_name)



def if_else_traverse(if_body,assign_block_list,const_func_name):
    if_body_list=if_body.body
    # else_body_list=if_body.orelse
    if len(if_body_list)==1:
        if_body_flag =whether_fun_is_append(if_body_list[0],assign_block_list,const_func_name)
    else:
        return False
    #print("if body flag: ",if_body_flag)

    return if_body_flag and else_traverse(if_body,assign_block_list,const_func_name)
def if_traverse(one_body,assign_block_list,const_func_name):
    orelse = one_body.orelse
    if_body=one_body.body
    if not orelse:
        return for_traverse(one_body,assign_block_list,const_func_name)
    else:
        #print("it is if-else node: ", ast.unparse(one_body))
        return if_else_traverse(one_body,assign_block_list,const_func_name)



def for_traverse(node,assign_block_list,const_func_name):
    for_body_list = node.body
    if isinstance(node, ast.For) and node.orelse:
       # print("come here")
        return False
    if len(for_body_list) == 1:
        one_body=for_body_list[0]
        if isinstance(one_body, ast.For):
            #print("come here for_traverse")
            return for_traverse(one_body,assign_block_list,const_func_name)
        elif isinstance(one_body, ast.If):
            #print("it is if node")
            return if_traverse(one_body,assign_block_list,const_func_name)

        elif isinstance(one_body, ast.Expr):
            #print("come here for_traverse and it is an Expr",whether_fun_is_append(one_body,assign_block_list))
            return whether_fun_is_append(one_body,assign_block_list,const_func_name)
        else:

            #print("the for/if sentences has other stmts so cannot transform such as try stmt", ast.unparse(one_body),type(one_body))
            return False
    else:
        return False
'''
beg1 end1
beg2 end2
'''

def filter_overlap(code_index_list):
    no_overlap_list=[]
    if len(code_index_list)==1:
        return code_index_list
    for i in range(len(code_index_list)):
        code=code_index_list[i]
        beg=code[0].lineno
        end=code[0].end_lineno
        for j in range(len(code_index_list)):
            if i==j:
                continue
            fuzhu_beg = code_index_list[j][0].lineno
            fuzhu_end = code_index_list[j][0].end_lineno
            if beg>=fuzhu_beg and end<=fuzhu_end:
                break
        else:
            no_overlap_list.append(code)

    return no_overlap_list
# def filter_overlap(code_index_list):
#     no_overlap_list=[]
#     if len(code_index_list)==1:
#         return code_index_list
#     for i in range(len(code_index_list)-1):
#         code1=code_index_list[i]
#         beg=code1[2]
#         end=code1[3]
#         for j in range(i+1,len(code_index_list)):
#             fuzhu_beg = code_index_list[j][2]
#             fuzhu_end = code_index_list[j][3]
#             if beg>=fuzhu_beg and end<=fuzhu_end:
#                 break
#         else:
#             no_overlap_list.append(code1)
#
#
#     return no_overlap_list
def is_use_var(next_child, vars):
    s = ast.unparse(next_child)
    if s != list(vars)[0]:
        g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
        for toknum, to_child, _, _, _ in g:
            if to_child.strip() == list(vars)[0]:
                return 1
    return 0
'''
1.首先for_node中除了对vars使用追加元素，不应该再被使用
    特别地有时候for_node中会包含这个文件中自定义的函数调用，此时我们需要获得这个函数代码，判断是否使用了vars
2. 找到将vars赋值为空的语句ass，然后判断ass与for_node之间是否使用了vars，这种情况也不可以被简化
还要判断ass所在层的在for_node后面的代码中是否使用了vars，此时决定着是否可以remove ass
特别的对于如果ass在更外层相比于for_node,此时如果ass所在层内中，for_node属于for节点的body此时也不可以化简为comprehension
'''
def whether_first_var_is_empty_assign(tree,for_node,vars,const_func_name="append",const_empty_list=["[]"]):
    assign_stmt = None
    assign_stmt_lineno = None
    ass_block_lineno = None
    ass_block_end_lineno = None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)

    def visit_single_vars(target, list_vars):
        # print(">>>>>>>target: ",target.__dict__)
        if isinstance(target, (ast.Name)):
            list_vars.append(ast.unparse(target))
        elif isinstance(target, ast.Subscript):
            list_vars.append(ast.unparse(target))
            visit_single_vars(target.value, list_vars)
        elif isinstance(target, ast.Attribute):
            # print("attr: ", ast.unparse(target))
            list_vars.append(ast.unparse(target))
            visit_single_vars(target.value, list_vars)
        # else:
        #     print("visit_single_vars else node: ", ast.unparse(target))
        #     for e in ast.iter_child_nodes(target):
        #         print("visit_single_vars e: ", e, ast.unparse(e))
        #         visit_single_vars(e, list_vars)
    def get_time_var(vars):
        all_var_list = []
        for ind,e_var in enumerate(ast.walk(ast.parse(list(vars)[0]))):

            if ast.unparse(e_var)==list(vars)[0] and isinstance(e_var,(ast.Subscript,ast.Attribute,ast.Name)):
                # print("var: ",e_var,ast.unparse(e_var))
                visit_single_vars(e_var, all_var_list)
                break
        return all_var_list
    def whether_contain_var(node,vars,time_var_list):
        count=0

        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                # print("child: ",child)
                if child in time_var_list:#== list(vars)[0]:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time_var_list=get_time_var(vars)
    remove_ass_flag = 0
    time=whether_contain_var(for_node, vars,time_var_list)
    # print("time: ",time)
    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child, ast.Call):
            a = []
            # if whether_fun_is_append(child, a, const_func_name):
            #     append_time+=len(time_var_list)
            append_time += whether_fun_is_append(child, a, const_func_name)

            def is_call_use_vars(tree, fun_name, time_var_list):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun, "name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) in time_var_list:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, time_var_list)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt, remove_ass_flag
        # time += whether_contain_var(child, vars)
    # print("append_time: ", append_time)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt, remove_ass_flag
            # break
    for_node_block_record=None
    for node in ast.walk(tree):
        # print(">>>>> walk the node is: ", node,ast.unparse(node))
        # if not hasattr(node,"lineno"):
        #     print("yes",node)
        #     continue

        if  hasattr(node,"lineno") and node.lineno==for_node.lineno:
            # print("come the for_node")
            break
        # for child in ast.iter_child_nodes(node):
        if hasattr(node,'body') and isinstance(node.body, list) and \
            ((hasattr(node,'lineno') and node.lineno<for_node.lineno<=node.end_lineno) or (hasattr(node.body[0],'lineno') and node.body[0].lineno<for_node.lineno<node.body[-1].end_lineno)):

            # print(">>>>> the node is: ",ast.unparse(node))
            for child in node.body:
                # print("come here node: ",child,ast.unparse(child),isinstance(child,ast.Expr))

                if isinstance(child, ast.Assign):
                    if ast.unparse(child).strip().split("=")[0].strip() == list(vars)[0] and ast.unparse(child.value) in const_empty_list:
                        assign_stmt = child
                        assign_stmt_lineno = child.lineno
                        ass_block_end_lineno=node.end_lineno if hasattr(node,'lineno') else node.body[0].lineno
                        # if ass_block_end_lineno < for_node.lineno:
                        #     remove_ass_flag = 0

                        for_node_block_record = node
                        flag = 2


                # elif assign_stmt_lineno:
                #     for child_child in ast.walk(child):
                #         if child_child==for_node:
                #             for_node_block_record = node
                #             break
    # print("whether has assign: ",assign_stmt_lineno)
    if  for_node_block_record:
        # print(">>>>>>>come here for_node_block_record:\n ",ast.unparse(for_node_block_record))
        for child in ast.iter_child_nodes(for_node_block_record):#.body:
            if child==for_node:
                remove_ass_flag=1
        for node in ast.walk(for_node_block_record):
            # 在ass和for_node之间不可以使用变量vars
            if hasattr(node,"end_lineno") and hasattr(node,"lineno"):
                if node != assign_stmt and node.end_lineno < for_node.lineno and node.lineno > assign_stmt_lineno:
                    s = ast.unparse(node)

                    if s != list(vars)[0]:
                        g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                        for toknum, child, _, _, _ in g:
                            if child.strip() in time_var_list:# list(vars)[0]:
                                flag = 1
                                # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
                                return flag, assign_stmt_lineno, assign_stmt, remove_ass_flag
            if isinstance(node, (ast.For, ast.While,
                                           ast.AsyncFor)) and for_node.lineno>node.lineno > assign_stmt.lineno and node.end_lineno >= for_node.end_lineno:

                    # print(">>>>>>>>come here ")
                    return 1, assign_stmt_lineno, assign_stmt, remove_ass_flag

        #         '''
        #         elif assign_stmt_lineno:
        #             '''
        #             如果ass和for_node在同一层则不需要去除，
        #             如果ass的层数在更外层，则我们需要看这一层后面是否使用了vars 如果使用了vars则ass的赋值为空的语句不可以删除
        #             如果ass在更外层，在ass所在层的后面节点中，如果存在for节点，并且for节点包含该for_node，则如果for_node后使用了vars，该for_node仍然不可以简化。
        #             ass=[]
        #             for :
        #                 for :
        #                     ass.append
        #             '''
        #             # print("ass_block_end_lineno: ",ass_block_end_lineno,for_node.lineno)
        #
        #             if isinstance(child,ast.For) and child.lineno==for_node.lineno:
        #                 flag = 2
        #                 break
        #                 # return 2, assign_stmt_lineno, assign_stmt, remove_ass_flag
        #
        #             for next_child in ast.walk(child):
        #                 if not hasattr(next_child, "lineno"):
        #                     continue
        #                 if isinstance(next_child,(ast.For,ast.While,ast.AsyncFor)) and next_child.lineno<for_node.lineno and next_child.end_lineno>=for_node.end_lineno:
        #                     # print(">>>>>>>>come here ")
        #                     return 1, assign_stmt_lineno, assign_stmt, remove_ass_flag
        #
        #                     # for for_child in ast.walk(next_child):
        #                         # if hasattr(for_child,'lineno'):
        #                         #     if for_child.lineno>for_node.end_lineno:
        #                         #         if is_use_var(for_child,vars):
        #                         #             flag=1
        #                         #             print(">>>>>>>>come here ")
        #                         #             return flag, assign_stmt_lineno, assign_stmt, remove_ass_flag
        #                 elif  next_child.lineno>for_node.end_lineno:
        #                     if is_use_var(next_child, vars):
        #                         remove_ass_flag = 0
        #                         # break
        #             return flag,assign_stmt_lineno, assign_stmt, remove_ass_flag
        #         '''
        # # 在ass和for_node之间不可以使用变量vars
        # if assign_stmt_lineno and node!=assign_stmt and node.end_lineno<for_node.lineno and node.lineno>assign_stmt_lineno :
        #     s=ast.unparse(node)
        #
        #     if s!=list(vars)[0]:
        #         g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
        #         for toknum, child, _, _, _ in g:
        #             if child.strip() == list(vars)[0]:
        #                 flag = 1
        #                 # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
        #                 break
        #         if flag==1:
        #             break
        # 判断跟compreh不属于同一个父母的在for_node后面的代码片段是否使用了变量vars
        # if assign_stmt_lineno and node!=assign_stmt and node.lineno>ass_parent.end_lineno:
        #     s = ast.unparse(node)
        #
        #     if s != list(vars)[0]:
        #         g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
        #         for toknum, child, _, _, _ in g:
        #             if child.strip() == list(vars)[0]:
        #                 remove_ass_flag = 0
        #                 print("no we cannot remove the ass: ","\n>>>>>>>>>",ast.unparse(for_node),"\n>>>>>>>>>",ast.unparse(node))
        #                 break
        #         if remove_ass_flag == 0:
        #             break
    else:
        return 1,assign_stmt_lineno,assign_stmt,remove_ass_flag
    return flag,assign_stmt_lineno,assign_stmt,remove_ass_flag
'''
def whether_first_var_is_empty_assign(tree,for_node,vars,const_func_name="append",const_empty_list=["[]"]):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)


    def whether_contain_var(node,vars):
        count=0
        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                if child == list(vars)[0]:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time=whether_contain_var(for_node, vars)
    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child, ast.Call):
            a = []
            append_time += whether_fun_is_append(child, a,const_func_name)

            def is_call_use_vars(tree, fun_name, vars):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun, "name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) == list(vars)[0]:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, vars)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt
        # time += whether_contain_var(child, vars)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt
            # break
    for node in ast.walk(tree):
        if not hasattr(node,"lineno"):
            continue

        if node.lineno==for_node.lineno:
            break
        if isinstance(node, ast.Assign):
            # print("the assign: ",ast.unparse(node),ast.unparse(node).strip().split("=")[0].strip())
            if ast.unparse(node).strip().split("=")[0].strip()==list(vars)[0] and ast.unparse(node.value) in const_empty_list:
                assign_stmt=node
                assign_stmt_lineno=node.lineno
                flag = 2
        if assign_stmt_lineno and node.end_lineno<for_node.lineno and node.lineno>assign_stmt_lineno :
            s=ast.unparse(node)

            if s!=list(vars)[0]:
                g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                for toknum, child, _, _, _ in g:
                    if child.strip() == list(vars)[0]:
                        flag = 1
                        # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
                        break
                if flag==1:
                    break
    return flag,assign_stmt_lineno,assign_stmt
'''
'''
def whether_first_var_is_empty_assign(tree,for_node,vars,const_empty_list=["[]"]):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # print("the var: ",vars)
    for node in ast.walk(tree):
        if not hasattr(node,"lineno"):
            continue

        if node.lineno==for_node.lineno:
            break
        if isinstance(node, ast.Assign):
            # print("the assign: ",ast.unparse(node),ast.unparse(node).strip().split("=")[0].strip())
            if ast.unparse(node).strip().split("=")[0].strip()==list(vars)[0] and ast.unparse(node.value) in const_empty_list:
                assign_stmt=node
                assign_stmt_lineno=node.lineno
                flag = 2
        if assign_stmt_lineno and node.lineno < for_node.lineno and node.lineno > assign_stmt_lineno:
            for child in ast.walk(node):
                if ast.unparse(child).strip().split("=")[0].strip() == list(vars)[0]:
                    flag = 1
                    break
            else:
                continue
            if flag == 1:
                break


    return flag,assign_stmt_lineno,assign_stmt
'''
def get_complicated_for_comprehen_code_list(tree,content,const_empty_list=["[]"],const_func_name="append"):
    #code1 = ast.unparse(tree)
    code_index_start_end_list = []

    start_lineno=tree.body[0].lineno
    flag_occur_for=None
    #code1 = "\n".join(content.split("\n")[start_lineno - 1:])
    for ind_line,node in enumerate(ast.walk(tree)):

            #print("start_line: ",node.lineno)
        if isinstance(node, ast.For):
            flag_occur_for=node
            #print("node_line: ",ast.unparse(node),node.lineno, node.end_lineno)
            # if node.lineno!=1674:
            #     continue

            assign_block_list=[]
            #print("it is what: ",for_traverse(node,assign_block_list))
            if for_traverse(node,assign_block_list,const_func_name):#判断for语句是否满足For_Match
                #print("maybe comrephension")
                '''
                判断是否是同一个变量的append
                '''
                vars = set([])
                for each_block in assign_block_list:
                    pre_name,call_name=get_func_name(each_block[0])
                    vars.add(pre_name)
                if len(vars)!=1:
                    continue
                # print("for node: ",ast.unparse(node),node.lineno)
                '''
                判断是否是同一个变量的append
                '''
                flag,assign_stmt_lineno,assign_stmt,remove_ass_flag=whether_first_var_is_empty_assign(tree, node, vars,const_func_name,const_empty_list)
                if flag!=2:
                    print("it does not have the var",vars,flag)
                    continue
                #print("flag: ", assign_stmt,assign_stmt_lineno,suite)
                assign_block_list_str = []
                for one_block in assign_block_list:
                    one_block_str = []
                    for e in one_block:
                        one_block_str.append(ast.unparse(e))
                    assign_block_list_str.append(one_block_str)
                # print(">>>>>>>>>>>>>>>>>>>>>>it is complicated comprehension code1\n", ast.unparse(assign_stmt),"\n",ast.unparse(node),node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str,assign_stmt)
                # print(assign_stmt+"\n"+ast.unparse(node))
                code_index_start_end_list.append([node,assign_stmt,remove_ass_flag])#,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])
                # code_index_start_end_list.append([ast.unparse(node),node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str, assign_stmt])
    #print("len: ", len(code_index_start_end_list))
    code_index_start_end_list = filter_overlap(code_index_start_end_list)
    #print("len: ",len(code_index_start_end_list))

    return code_index_start_end_list


def save_one_repo(repo_name,save_complicated_code_dir_pkl):
    count=0
    one_repo_chained_comparison_code_list = []
    dict_file = dict()
    for file_info in dict_repo_file_python[repo_name]:
        file_path = file_info["file_path"]
        file_html = file_info["file_html"]
        # if file_html!="https://github.com/bndr/pipreqs/tree/master/pipreqs/pipreqs.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":#"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#:#"https://github.com/aws/aws-cli/tree/master/awscli/customizations/s3/subcommands.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":##"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/common.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":##"https://github.com/HypothesisWorks/hypothesis/tree/master/hypothesis-python/examples/test_rle.py":
        #     continue
        # if file_html!="https://github.com/HypothesisWorks/hypothesis/tree/master/hypothesis-python/examples/test_rle.py":
        #     continue
        # print("file_html: ", file_html)
        try:
            content = util.load_file_path(file_path)
        except:
            print(f"{file_path} is not existed!")
            continue

        try:
            file_tree = ast.parse(content)
            ana_py = ast_util.Fun_Analyzer()
            ana_py.visit(file_tree)
            # one_file_chained_comparison_code_list = []
            #print("func number: ",file_html,len(ana_py.func_def_list))
            dict_class = dict()
            for tree, class_name in ana_py.func_def_list:
                # if 1:
                #print("come here",ast.unparse(tree))
                new_code_list = get_complicated_for_comprehen_code_list(tree,content)
                # one_file_chained_comparison_code_list.extend(get_complicated_for_comprehen_code_list(tree,content))
                ast_util.set_dict_class_code_list(tree, dict_class, class_name, new_code_list)

                # count+=len(one_file_chained_comparison_code_list)
            dict_file[file_html] = dict_class
            #print(dict_class)
            # if one_file_chained_comparison_code_list:
            #     one_repo_chained_comparison_code_list.append(
            #         [one_file_chained_comparison_code_list, file_path, file_html])
                # print("one_file_truth_value_test_code_list: ",one_repo_chained_comparison_code_list)
                    # break
        except SyntaxError:
            print("the file has syntax error")
            continue
        except ValueError:
            print("the file has value error: ", content, file_html)
            continue
        # break
    #print("count: ",count)
    # util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
    print("save successfully! ", count,save_complicated_code_dir_pkl + repo_name)

    # # print(one_repo_chained_comparison_code_list[0],count_complicated_code)
    #
    #     # print(one_repo_chained_comparison_code_list)
    #
    #     util.save_json(save_complicated_code_dir, repo_name, one_repo_chained_comparison_code_list)
    #     print("save successfully! ", count,save_complicated_code_dir + repo_name)
        # break


if __name__ == '__main__':
    code = '''
# a=[]
# for i in range(4):
#     a.append(i)
# a=[]
for i in range(3):
    call(a)
a=[]
b=[]
for i in range(4):
    if i>3:
        for j in range(5):
            # i=i**2
            # a.append(j)
            b.append(i)
#     # elif i>4:
#     #     a.append(i)
# for i in range(4):
#      if i>3:
#          if i>3:
#              a.append(i)
#          else:
#               a.append(i)

for i in range(4):
     # if i>3:
         if i>3:
             a.append(i)
         else:
              a.append(i)
# a=[]
for i in range(3):
    b=[]
    for i in range(3): # 这个节点里包含 除了assign和Call的节点 因为在comprehension的繁杂代码中不可以再有for语句
         if i>3:
             b.append(i)
    a.append(b)  
threads = []
for _ in range(4):
    threads.append(threading.Thread(target=test)) 
'''

    '''

    tree = ast.parse(code1)
    code_list=get_complicated_for_comprehen_code_list(tree, code1)
    #code_list=get_complicated_for_comprehen_code_list(code1)
    print("len: ",len(code_list))
    for e in code_list:
        print(">>>>>code1： ", e[0])
        print("block: ", e[-1])
        # for each in  e[-1]:
        #     print(each)

        print("----------END------------")
    # test_code="a=[]"
    # test_tree = ast.parse(test_code)
    # print(ast.unparse(test_tree))
    '''
    # layer_node_list = []
    # ast_util.extract_ast_cur_layer_node(tree, layer_node_list)
    '''
    get comprehension node
    Block=> Assign;...; For
    For=>”for” ( For| If|If-else|body)
    If=>”if” (body| For| If| If-else)
    If-else=>”if” body “else” (body| If-else)
    body=> (Assign|Call)*;Append*
    '''
    # 对于

    save_complicated_code_dir_pkl = util.data_root + "complicated_code_dir_pkl/for_compre_list/"

    save_complicated_code_dir = util.data_root + "complicated_code_dir/for_comrephension_list_complicated_only_one_stmt/"
    #'''
    #dict_repo_file_python = util.load_json(util.data_root, "python3_repos_files_info")
    dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")

    # repo_name_list=[]
    # for repo_name in dict_repo_file_python:
    #
    #     repo_name_list.append(repo_name)
    #     if repo_name=="hosts":
    #         print("Come here repo")
    #         save_one_repo(repo_name)

    # print("repo num: ", len(list(dict_repo_file_python.keys())))
    # count_complicated_code = 0
    # count_repo = 0
    repo_list = []
    for ind, repo_name in enumerate(dict_repo_file_python):
        if repo_name!="pipreqs":#"proselint":#"networkx":#"nni":#"aws-cli":#"youtube-dl":#"localstack":#"transitions":#"youtube-dl":#"hypothesis":
            continue
        # if repo_name!="hypothesis":
        #     continue
        # if repo_name!="Real-Time-Voice-Cloning":#"keras-bert":#"Legofy":
        #     continue
        #print("repo_name: ", repo_name)
        repo_list.append(repo_name)
    print("repo num: ", len(repo_list))
    save_one_repo(repo_list[0])
    '''
    pool = newPool(nodes=30)
    pool.map(save_one_repo, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    '''

    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    file_list = set([])
    repo_code_num = dict()
    result_compli_for_else_list = []
    all_count_repo, all_file_count, all_me_count = 0, 0, 0
    for file_name in os.listdir(save_complicated_code_dir_pkl):
        all_count_repo += 1
        repo_name = file_name[:-4]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

        repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
            complicate_code)
        # for code_list, file_path, file_html in complicate_code:
        code_count += repo_code_count
        file_count += repo_file_count
        me_count += repo_me_count
        all_file_count += repo_all_file_count
        all_me_count += repo_all_me_count
        repo_exist = 0
        for file_html in complicate_code:
            for cl in complicate_code[file_html]:
                for me in complicate_code[file_html][cl]:
                    if complicate_code[file_html][cl][me]:
                        repo_exist = 1
                        for code in complicate_code[file_html][cl][me]:
                            # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                            #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])

                            result_compli_for_else_list.append(
                                [repo_name, file_html, cl, me, ast.unparse(code[0]), ast.unparse(code[1])])

            # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
        count_repo += repo_exist

    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    util.save_csv(util.data_root + "complicated_code_dir_pkl/for_else.csv", result_compli_for_else_list,
                  ["repo_name", "file_html","class_name", "me_name", "for_code", "assign_code"])

    '''
    count = 0
    result_compli_for_else_list = []
    for file_name in os.listdir(save_complicated_code_dir):
        complicate_code = util.load_json(save_complicated_code_dir, file_name[:-5])
        for each_file in complicate_code:

            # for code_list, file_path,file_html in each_file:
            #
            #     print("count: ",code_list)
            code_list = each_file[0]
            file_path = each_file[1]
            file_html = each_file[2]
            count += len(code_list)
            # print("count: ", count)
            for code1 in code_list:
                #print("code1: ",code1)
                repo_name = file_html.split("/")[4]
                #print("code1: ",code1)
                result_compli_for_else_list.append(
                    [repo_name, code1[-2]+"\n"+ast.unparse()code1[0]+"\n"+code1[-1], str(code1[1])+" "+str(code1[2])+" "+str(code1[3]), file_html, file_path])

        #     print("one code1: ",repo_name,code1,file_html,file_path)
        #     break
        # break
        # print("file: ",file_name)
        # break
    print("count: ", count, len(os.listdir(save_complicated_code_dir)))#,result_compli_for_else_list)
    # util.save_csv(util.data_root + "complicated_code_dir/for_comrephension_list_complicated_only_one_stmt.csv",
    #               result_compli_for_else_list, ["repo_name", "code1", "start_end_assign_ind", "file_html", "file_path"])
    '''














import sys,ast

# sys.path.append("/Users/zhangzejunzhangzejun/PycharmProjects/pythonProject/python-ast-explorer-master/extract_transform_complicate_code_new/")
import copy
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
# from code1.extract_simp_cmpl_data import ast_util
import traceback
# from pathos.multiprocessing import ProcessingPool as newPool


def get_pos(ele, one_var):
    if one_var[0] == ele:
        return "l"
    else:
        return "r"

def merge(node1, node2):
    ops = node2.ops
    comparator = node2.comparators

    for op,e in zip(ops,comparator):
        node1.ops.append(op)
        node1.comparators.append(e)

    #print("code1 list: ",code1, ast.unparse(node2),ast.unparse(node1))
    return ast.unparse(node1)
def change_compare_node(node1):
    new_node = copy.deepcopy(node1)
    new_node.left = node1.comparators[-1]
    new_node.comparators = node1.comparators[::-1][1:]
    # new_node.comparators.extend(node1.comparators[:-1][::-1])
    new_node.comparators.append(node1.left)
    for ind, op in enumerate(node1.ops[::-1]):
        if isinstance(op, ast.Gt):
            new_node.ops[ind] = ast.Lt()
            # new_node.ops.append(ast.Lt)
        elif isinstance(op, ast.Lt):
            new_node.ops[ind] = ast.Gt()
            # new_node.ops.append(ast.Gt)
        elif isinstance(op, ast.GtE):
            new_node.ops[ind] = ast.LtE()
            # new_node.ops.append(ast.LtE)
        elif isinstance(op, ast.LtE):
            new_node.ops[ind] = ast.GtE()
            # new_node.ops.append(ast.GtE)
        else:
            continue
    return new_node

def transform_left_right(pos1, pos2, node1, node2):
    code = ""
    # when  timeout is not None and 3-2>timeout it cannot transformed 3-2>timeout is not None
    # 这里是处理
    # print("pos1, pos2, node1, node2: ",pos1, pos2, ast.unparse(node1), ast.unparse(node2))
    # 把判断是否为None的node都放在第一个节点node1，并且node1.left=None,因为这种情况下我们不能对node1进行移动或者交换
    node1_still_flag=0

    if isinstance(node1.ops[0],(ast.Is,ast.IsNot,ast.Eq,ast.NotEq)):
        if pos1=='l' and ast.unparse(node1.comparators[0])=='None':
            node1.left, node1.comparators[0] = node1.comparators[0], node1.left
            pos1 = 'r'
            node1_still_flag=1
    elif isinstance(node2.ops[0],(ast.Is,ast.IsNot,ast.Eq,ast.NotEq)):
        if pos2=='l' and ast.unparse(node2.comparators[0])=='None':
            node2.left, node2.comparators[0] = node2.comparators[0], node2.left
            node1,node2=node2,node1
            pos2 = 'r'
            pos1,pos2=pos2,pos1
            node1_still_flag = 1


    if pos1 == 'r' and pos2 == 'l':  # 右左
        # 不变
        code = merge(node1, node2)
        # print("c->s: ", code1)
        pass
    elif pos1 == 'l' and pos2 == 'r':  # 左右
        if node1_still_flag:
            return ""
        code = merge( node2,node1)
        # 交换两个节点
        pass
    elif pos1 == 'l' and pos2 == 'l':  # 左左
        # change node1
        if isinstance(node1.ops[-1],(ast.NotIn,ast.In)) and isinstance(node2.ops[-1],(ast.NotIn,ast.In)) :
            return ""
        elif isinstance(node1.ops[-1],(ast.NotIn,ast.In)):
            if node1_still_flag:
                return ""
            new_node = change_compare_node(node2) # change node2 变成右
            code = merge(new_node,node1)
        else:
            new_node = change_compare_node(node1)
            code = merge(new_node, node2)
            pass
        # new_node=change_compare_node(node1)
        # code1 = merge(new_node, node2)
    else:  # 右右
        #print("come here")
        if isinstance(node1.ops[-1],(ast.NotIn,ast.In)) and isinstance(node2.ops[-1],(ast.NotIn,ast.In)) :
            return ""
        elif isinstance(node2.ops[-1],(ast.NotIn,ast.In)):
            #print("come here")
            if node1_still_flag:
                return ""
            # if isinstance(node1.ops[-1],(ast.Is, ast.IsNot, ast.Eq, ast.NotEq)) and ast.unparse(node1.left)=="None":
            #     return ""
            new_node = change_compare_node(node1) # change 左
            code = merge(node2, new_node)
        else:
            #print("come here")
            new_node = change_compare_node(node2)
            code = merge(node1, new_node)
            pass
    return code


def transform_remain_code(remain_vars_list, ignore_ind_list):
    code = []
    flag=0# 0 表示应该放在后面，1表示应该放在前面
    for ind, e in enumerate(remain_vars_list):
        if ind in ignore_ind_list:
            continue
        if ind==0:
            flag=1
        code.append(ast.unparse(e[1]))
    # code1.append(ast.unparse(remain_vars_list[-1]))
    return " and ".join(code),flag
def transform_chained_comparison(node):
    if not (isinstance(node,ast.BoolOp) and isinstance(node.op,ast.And)):
        return 0,node

    vars_list = []
    values = node.values
    other_code = []
    for value in values:
        if isinstance(value, ast.Compare):
            left = value.left
            comparator = value.comparators[-1]
            vars = [[ast.unparse(left), ast.unparse(comparator)], value]
            vars_list.append(vars)
        else:
            other_code.append(ast.unparse(value))
    if other_code:

        other_code =  " and ".join(other_code)
    else:
        other_code = ""

    remain_vars_list = copy.deepcopy(vars_list)
    overlap_ind_list = []
    for ind, one_var in enumerate(vars_list):
        if ind in overlap_ind_list:
            continue

        for ind_next in range(ind + 1, len(vars_list)):
            if ind_next in overlap_ind_list:
                continue
            another_var = vars_list[ind_next]
            if set(one_var[0]) & set(another_var[0]):
                ele = list(set(one_var[0]) & set(another_var[0]))[0]
                # print("intersect var: ",ele)

                one_pos = get_pos(ele, one_var[0])
                another_pos = get_pos(ele, another_var[0])

                # print("code1 of node1 and node2: ",ast.unparse(one_var[1]),ast.unparse(another_var[1]))
                transform_code = transform_left_right(one_pos, another_pos, one_var[1], another_var[1])
                if not transform_code:
                    # print("come here")
                    return 0,node

                remain_code,pos_flag = transform_remain_code(remain_vars_list, [ind, ind_next])
                if remain_code:
                    if pos_flag:
                        remain_code = remain_code+" and "+transform_code
                    else:
                        remain_code=transform_code+" and "+remain_code
                else:
                    remain_code=transform_code
                if other_code:
                    other_code=" and "+other_code

                # print("all_code: ", transform_code, remain_code, other_code)
                # all_node=transform_code,remain_code,other_code]
                for node in ast.walk(ast.parse(remain_code  + other_code)):
                    if isinstance(node,ast.BoolOp) and isinstance(node.op, ast.And):

                        return 1, node
                    elif isinstance(node,ast.Compare):
                        return 1,node
            # else:
            #     return 0,node

    return 0,node
# 对找到的代码片段不断进行转换直至不可以转换
def check_chained_comparison(tree,new_code_list):
    for node in ast.walk(tree):
        if isinstance(node, ast.BoolOp):
            op = node.op
            old_node=copy.deepcopy(node)
            if isinstance(op, ast.And):
                flag_code=0
                while 1:
                    flag,new_node = transform_chained_comparison(node)
                    # if new_node
                    if not flag:
                        break
                    else:
                        flag_code=1
                    node=new_node
                if flag_code:
                    # print(f"old_node:{ast.unparse(old_node)}, new_node:{ast.unparse(new_node)} ")
                    new_code_list.append([old_node,new_node])

def get_chain_compare(content):
    code_pair_list=[]
    # print(content)
    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)
        # print("come here",ana_py.func_def_list)
        for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = []
            check_chained_comparison(tree, new_code_list)

            for old_node,new_node in new_code_list:
                old_node_content = ast.get_source_segment(content, old_node)

                line_list=[[(old_node.lineno,old_node.col_offset), (old_node.end_lineno,old_node.end_col_offset)]]

                code_pair_list.append([class_name,me_name,[old_node_content],[ast.unparse(new_node)],line_list])
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def get_part_chain_compare(content):
    code_pair_list=[]
    # print(content)
    try:

        tree = ast.parse(content)
        # ana_py = ast_util.Fun_Analyzer()
        # ana_py.visit(file_tree)
        # print("come here",ana_py.func_def_list)
        if 1:
        # for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = []
            check_chained_comparison(tree, new_code_list)

            for old_node,new_node in new_code_list:
                old_node_content = ast.get_source_segment(content, old_node)

                line_list=[[(old_node.lineno,old_node.col_offset), (old_node.end_lineno,old_node.end_col_offset)]]

                code_pair_list.append(['',me_name,[old_node_content],[ast.unparse(new_node)],line_list])
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list
# def save_repo_for_else_complicated(repo_name):
#     start=time.time()
#     count_complicated_code = 0
#     print("come the repo: ", repo_name)
#     dict_file=dict()
#
#
#     for ind,file_info in enumerate(dict_repo_file_python[repo_name]):
#         # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
#         #     continue
#         file_path = file_info["file_path"]
#         file_html = file_info["file_html"]
#         # if file_html!="https://github.com/chainer/chainerrl/tree/master/chainerrl/experiments/train_agent_batch.py":#"https://github.com/alteryx/featuretools/tree/master/featuretools/synthesis/encode_features.py":#"https://github.com/horovod/horovod/tree/master/test/utils/common.py":#
#         #     continue
#         # if file_html != "https://github.com/indigo-dc/udocker/tree/master/udocker/engine/runc.py":
#         #     continue
#         try:
#             content = util.load_file_path(file_path)
#         except:
#             print(f"{file_path} is not existed!")
#             continue
#         print("come the file_html: ",file_html)
#         try:
#             file_tree = ast.parse(content)
#             ana_py = ast_util.Fun_Analyzer()
#             ana_py.visit(file_tree)
#
#             dict_class=dict()
#             for tree, class_name in ana_py.func_def_list:
#
#                 # print("tree_ func_name",tree.__dict__)
#                 new_code_list = []
#                 check_chained_comparison(tree, new_code_list)
#                 ast_util.set_dict_class_code_list(tree,dict_class, class_name, new_code_list)
#
#
#             dict_file[file_html]=dict_class
#
#         except SyntaxError:
#             print("the file has syntax error")
#             continue
#         except ValueError:
#             traceback.print_exc()
#             print("the file has value error: ", file_html)  # content,
#             continue
#         except:
#             traceback.print_exc()
#             print("the file has other error: ", file_html)
#             # break
#     end = time.time()
#     #'''
#     if dict_file:
#         # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
#         util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#         # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#         print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
#     else:
#         print(end-start," the repo has no for else")
#         util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#         # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)

if __name__ == '__main__':
    code='''

a>b and a>c and 1>=c # and c>1 and e
#len(cur_info['cont_end_pts']) == 0 and len(other_infos) == 0
#edge_canvas[x, y] != -1 and edge_canvas[x, y] != valid_edge_id
'up' in direc.lower() and direc.lower()>1
#'up' in direc.lower() and 1>direc.lower()
#'up' in direc.lower() and direc.lower() in a
#direc.lower() in a and 'up' not in direc.lower()
#a>1  and  direc.lower() in a 左，右
#1<a  and  direc.lower() in a 右，右
#1<a  and  a in direc.lower() #右，左
#b in a  and  a in direc.lower()  #右，左
b in a  and  direc.lower() in a
log_interval is not None and t >= log_interval and t % log_interval < num_envs
'''

    # save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/chain_comparison/"
    #
    # dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")
    # all_start_time = time.time()
    # '''
    # new_code_list=[]
    # tree = ast.parse(code1)
    # check_chained_comparison(tree, new_code_list)
    # '''
    #
    # #'''
    # repo_list = []
    # for ind, repo_name in enumerate(dict_repo_file_python):
    #     # if repo_name!="udocker":#"chainerrl":##"horovod" and  repo_name!="featuretools" :#"featuretools":#featuretools
    #     #     continue
    #     # print("repo_name: ", repo_name)
    #     # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
    #     #
    #     #     continue
    #     repo_list.append(repo_name)
    # # save_repo_for_else_complicated(repo_list[0])
    # '''
    #
    # print("len: ", len(repo_list))
    # pool = newPool(nodes=20)
    # pool.map(save_repo_for_else_complicated, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    # pool.close()
    # pool.join()
    # '''
    #
    # files_num_list = []
    # star_num_list = []
    # contributor_num_list = []
    # count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    # file_list = set([])
    # repo_code_num = dict()
    # result_compli_for_else_list = []
    # all_count_repo, all_file_count, all_me_count = 0, 0, 0
    # for file_name in os.listdir(save_complicated_code_dir_pkl):
    #     all_count_repo += 1
    #     repo_name = file_name[:-4]
    #     # files_num_list.append(repo_files_info[repo_name])
    #     # star_num_list.append(repo_star_info[repo_name])
    #     # contributor_num_list.append(repo_contributor_info[repo_name])
    #
    #     complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)
    #
    #     repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
    #         complicate_code)
    #     # for code_list, file_path, file_html in complicate_code:
    #     code_count += repo_code_count
    #     file_count += repo_file_count
    #     me_count += repo_me_count
    #     all_file_count += repo_all_file_count
    #     all_me_count += repo_all_me_count
    #     repo_exist = 0
    #     for file_html in complicate_code:
    #         for cl in complicate_code[file_html]:
    #             for me in complicate_code[file_html][cl]:
    #                 if complicate_code[file_html][cl][me]:
    #                     repo_exist = 1
    #                     for code in complicate_code[file_html][cl][me]:
    #                         # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
    #                         result_compli_for_else_list.append(
    #                             [repo_name,file_html,  cl, me, code[0].lineno,ast.unparse(code[0]), ast.unparse(code[1])])
    #
    #         # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
    #     count_repo += repo_exist
    #     #     break
    #     # break
    #     # if complicate_code:
    #     #     repo_code_num[repo_name]=count_code_fra
    #     #     print(f"{repo_name} has  {count_code_fra} code1 fragments")
    #     # break
    # # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # # print(a)
    # # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    # print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # # 1 156 2943 100 1 2990 40102 salt
    # # 791 2010 103165 1291 800 121348 1192868
    # # util.save_csv(util.data_root + "complicated_code_dir_1000_star/chain_comparison.csv", result_compli_for_else_list,
    # #               ["repo_name", "class_name", "me_name", "old_code", "new_code", "file_html", "file_path"])
    # import random
    #
    # random.shuffle(result_compli_for_else_list)
    # util.save_csv(util.data_root + "result_csv/chain_compare.csv",
    #               result_compli_for_else_list[:400],
    #               ["repo_name", "file_html", "class_name", "me_name","line_no", "old_code", "new_code"])
    #
    # all_end_time = time.time()
    # print("total time: ", all_end_time - all_start_time)
    #
    #
    #
    #
    #
    #
    #










import argparse,os,sys,ast
import copy,shutil

abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path_1: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
# sys.path.append(pack_path)
sys.path.append("/".join(abs_path.split("/")[:-1]))
# sys.path.append("/".join(abs_path.split("/")[:-2]))
from RefactoringIdioms.extract_transform_complicate_code_new import \
    transform_for_else_compli_to_simple_improve_copy_result_csv, extract_compli_var_unpack_for_target_improve_new, \
    extract_compli_multiple_assign_code_improve_complete_improve
from RefactoringIdioms.extract_transform_complicate_code_new.comprehension import \
    extract_compli_for_comprehension_only_one_stmt_improve, extract_compli_for_comprehension_dict_only_one_stmt_new, \
    extract_compli_for_comprehension_set_only_one_stmt
from RefactoringIdioms.extract_transform_complicate_code_new import extract_compli_var_unpack_star_call_improve, \
    extract_compli_truth_value_test_code_remove_is_isnot, transform_chained_comparison_compli_to_simple
import util
import CodeInfo
# This is a sample Python script.

# Press ⌃R to execute it or replace it with your code.
# Press Double ⇧ to search everywhere for classes, files, tool windows, actions, and settings.
def save_output(outputpath):
    result=dict()
    util.save_file_path(outputpath,result)
def format_code_list(code_pair_list,filepath=''):
    dict_code_pair=dict()
    format_code_list=[]
    for  ind_idiom,(idiom,cl, me, oldcode, new_code,lineno_list) in enumerate(code_pair_list):
        code_info_cla = CodeInfo.CodeInfo(filepath, idiom,cl, me, oldcode, new_code,lineno_list)
        # print(code_info_cla.__dict__)
        print(f">>>Result{ind_idiom+1}",code_info_cla.full_info())
        format_code_list.append(code_info_cla)
        if idiom in dict_code_pair:
            dict_code_pair[ idiom].append(code_info_cla)
        else:
            dict_code_pair[idiom]=[code_info_cla]


        # format_code_list.append(code_info_cla.__dict__)
    return format_code_list,dict_code_pair
    # return code_dict
def get_list_comprehension(code_frag):
    print(">>>>>>>>>Checking List Comprehension")
    code_pair_list = extract_compli_for_comprehension_only_one_stmt_improve.get_list_compreh(code_frag)
    for e in code_pair_list:
        e.insert(0, "List Comprehension")
    return code_pair_list

def get_set_comprehension(code_frag):
    print(">>>>>>>>>Checking Set Comprehension")
    code_pair_list = extract_compli_for_comprehension_set_only_one_stmt.get_set_compreh(code_frag)

    for e in code_pair_list:
        e.insert(0, "Set Comprehension")
    return code_pair_list
def get_dict_comprehension(code_frag):
    print(">>>>>>>>>Checking Dict Comprehension")

    code_pair_list = extract_compli_for_comprehension_dict_only_one_stmt_new.get_dict_compreh(code_frag)
    for e in code_pair_list:
        e.insert(0, "Dict Comprehension")
    return code_pair_list
def get_chain_compare(code_frag):
    print(">>>>>>>>>Checking Chain Comparison")
    code_pair_list_chain_compare = transform_chained_comparison_compli_to_simple.get_chain_compare(code_frag)
    # code_pair_list.extend([e.insert(0,"Chain Compare") for e in code_pair_list_chain_compare])
    for e in code_pair_list_chain_compare:
        e.insert(0, "Chain Compare")
    return code_pair_list_chain_compare

def get_truth_value_test(code_frag):
    print(">>>>>>>>>Checking Truth Value Test")
    code_pair_list_truth_value = extract_compli_truth_value_test_code_remove_is_isnot.get_truth_value_test_code(
        code_frag)
    # code_pair_list.extend([e.insert(0,"Truth Value Test") for e in code_pair_list_truth_value])
    for e in code_pair_list_truth_value:
        e.insert(0, "Truth Value Test")
    return code_pair_list_truth_value


def get_ass_multi_targets(code_frag):
    print(">>>>>>>>>Checking Assign Multiple Targets")

    code_pair_list_assign_multi_targets = extract_compli_multiple_assign_code_improve_complete_improve.transform_multiple_assign_code(
        code_frag)
    # code_pair_list.extend([e.insert(0,"Assign Multi Targets") for e in code_pair_list_assign_multi_targets])
    for e in code_pair_list_assign_multi_targets:
        e.insert(0, "Assign Multi Targets")
    return code_pair_list_assign_multi_targets


def get_for_multi_targets(code_frag):
    print(">>>>>>>>>Checking For Multiple Targets")

    code_pair_list_for_multi_target = extract_compli_var_unpack_for_target_improve_new.transform_for_multiple_targets_code(
        code_frag)
    # code_pair_list.extend([e.insert(0,"For Multi Targets") for e in code_pair_list_for_multi_target])
    for e in code_pair_list_for_multi_target:
        e.insert(0, "For Multi Targets")
    return code_pair_list_for_multi_target


def get_for_else(code_frag):
    print(">>>>>>>>>Checking For Else")

    code_pair_listfor_else = transform_for_else_compli_to_simple_improve_copy_result_csv.transform_for_else_code(
        code_frag)
    for e in code_pair_listfor_else:
        e.insert(0, "For Else")
        # code_pair_list.append(e)
    return code_pair_listfor_else


def get_star_call(code_frag):
    # code_pair_list.extend([e.insert(0,"For Else") for e in code_pair_listfor_else])
    print(">>>>>>>>>Checking Star in Function Call")

    code_pair_list_call_star = extract_compli_var_unpack_star_call_improve.transform_star_call_code(
        code_frag)
    for e in code_pair_list_call_star:
        e.insert(0, "Call Star")
        # code_pair_list.append(e)
    return code_pair_list_call_star

def get_refactoring(idiom,code_frag,filepath):
    # print(f"************For File {filepath.split('/')[-1]}, begin to find refactorable non-idiomatic code with Python idioms************")
    #
    code_pair_list=[]
    # code_frag=util.load_file_path(file_path=filepath)
    if idiom == 'List Comprehension':
        code_pair_list = get_list_comprehension(code_frag)
        if code_pair_list:
            pass
            # print("code_list: ",code_pair_list,jsonify(code_pair_list).json[0][0])

        # return jsonify(code_pair_list)
        pass
    elif idiom == 'Set Comprehension':

        code_pair_list = get_set_comprehension(code_frag)
        if code_pair_list:
            pass
            # print("code_list: ", code_pair_list, jsonify(code_pair_list).json[0][0])

        # return jsonify(code_pair_list)
        pass
    elif idiom == 'Dict Comprehension':

        code_pair_list = get_dict_comprehension(code_frag)
        # return jsonify(code_pair_list)

        pass
    elif idiom == 'Chain Comparison':

        code_pair_list =get_chain_compare(code_frag)
        # return jsonify(code_pair_list)

        pass
    elif idiom == 'Truth Value Test':

        code_pair_list = get_truth_value_test(code_frag)
        # return jsonify(code_pair_list)

        pass
    elif idiom == 'Assign Multiple Targets':

        code_pair_list = get_ass_multi_targets(
            code_frag)
        # return jsonify(code_pair_list)

        pass
    elif idiom == 'For Multiple Targets':

        code_pair_list = get_for_multi_targets(
            code_frag)

        # return jsonify(code_pair_list)
        pass
    elif idiom == 'For Else':

        code_pair_list = get_for_else(
            code_frag)
        # return jsonify(code_pair_list)

        pass
    elif idiom == 'Star in Call':

        code_pair_list = get_star_call(
            code_frag)
        # return jsonify(code_pair_list)


        pass
    elif idiom == 'All':
        # idioms = ['All','List Comprehension', 'Set Comprehension', 'Dict Comprehension','Chain Comparison',
        # 'Truth Value Test','Assign Multiple Targets','For Multiple Targets','For Else'
        # ,'Star in Call']
        code_pair_list = []
        # print("code_frag")
        # print(code_frag)
        code_pair_list_compre = get_list_comprehension(code_frag)
        code_pair_list.extend(code_pair_list_compre)

        code_pair_set_compre=get_set_comprehension(code_frag)
        code_pair_list.extend(code_pair_set_compre)

        code_pair_dict_compre=get_dict_comprehension(code_frag)
        code_pair_list.extend(code_pair_dict_compre)

        code_pair_chain_compare = get_chain_compare(code_frag)
        code_pair_list.extend(code_pair_chain_compare)

        code_pair_truth_test = get_truth_value_test(code_frag)
        code_pair_list.extend(code_pair_truth_test)

        code_pair_multi_ass = get_ass_multi_targets(code_frag)
        code_pair_list.extend(code_pair_multi_ass)

        code_pair_for_multi_target=get_for_multi_targets(code_frag)
        code_pair_list.extend(code_pair_for_multi_target)

        code_pair_for_else = get_for_else(code_frag)
        code_pair_list.extend(code_pair_for_else)

        code_pair_star_call = get_star_call(code_frag)

        code_pair_list.extend(code_pair_star_call)

    new_code_list,dict_code_pair = format_code_list(code_pair_list,filepath)
    return new_code_list,dict_code_pair
def transform_line_no(code_list):
    dict_file_lineno_code=dict()
    new_code_list=[]

    for code in code_list:
        new_code_list.append(
            [code.lineno[0][0][0], code.lineno[-1][-1][0], code.compli_code,code.simple_code,code.idiom,code.lineno])
        #
        # for (e_lineno_start,e_col_start),(e_lineno_end,e_col_end) in code.lineno:
        #     # start_key=e_lineno_start+e_col_startj
        #     # end_key=float(f'{e_lineno_end}.{e_col_end}')
        #
        #     # key=(float(f'{e_lineno_start}.{e_col_start}'),float(f'{e_lineno_end}.{e_col_end}'))
        #     new_code_list.append([e_lineno_start,e_col_start,e_lineno_end,e_col_end,code.compli_code,code.simple_code,code.idiom])
        #     # dict_file_lineno_code[key]=[code.compli_code,code.simple_code,code.idiom]
    # sorted_new_code_list=sorted(new_code_list, key=lambda i: (i[0],i[1],-i[2], -i[3]))
    sorted_new_code_list=sorted(new_code_list, key=lambda i: (i[0],-i[1]))

    # print("sorted_new_code_list")
    # for e in sorted_new_code_list:
    #     print(e)
    return sorted_new_code_list
def replace_code(lineno_list,simp_code, code_fragment_list, bias=0):
    # lineno_list = chain_compare_code[-1]
    # # compli_code=chain_compare_code[-4]
    # simp_code = chain_compare_code[-3]
    new_code_fragment_list = []
    # print("simp_code: ", simp_code, code_fragment_list,lineno_list)
    pre_lineno = None
    for ind, ((star_row, star_col), (end_row, end_col)) in enumerate(lineno_list):
        star_row_new = star_row - bias - 1
        end_row_new = end_row - bias - 1
        replace_code = simp_code[ind]
        # print('replace_code: ',replace_code,star_row,star_col,end_row, end_col,code_fragment_list,star_row_new,end_row_new)
        if pre_lineno and pre_lineno[1] < star_row_new:  # len(new_code_fragment_list) < star_row_new:
            new_code_fragment_list += code_fragment_list[pre_lineno[1]+1:star_row_new]
        elif not pre_lineno and star_row_new>=1:
            new_code_fragment_list += code_fragment_list[:star_row_new]
        if pre_lineno:
            pre_lineno=max(star_row_new,pre_lineno[0]), max(end_row_new,pre_lineno[1])
        else:
            pre_lineno=star_row_new, end_row_new
        # pre_lineno=star_row_new,end_row_new
        if not replace_code:
            # new_code_fragment_list.append('')
            # print("each new_code_fragment_list: ", new_code_fragment_list,code_fragment_list[end_row_new + 1:])
            if ind == len(lineno_list) - 1:
                new_code_fragment_list += code_fragment_list[end_row_new + 1:]
            # print("each new_code_fragment_list: ", new_code_fragment_list,code_fragment_list[end_row_new + 1:])
            continue
        code_str = code_fragment_list[star_row_new][:star_col] + replace_code + code_fragment_list[end_row_new][
                                                                                end_col:]
        # print("each code_str: ",code_str)
        new_code_fragment_list.append(code_str)
        if ind == len(lineno_list) - 1:
            new_code_fragment_list += code_fragment_list[end_row_new + 1:]

        # print("each new_code_fragment_list: ", new_code_fragment_list)
    return new_code_fragment_list
def file_replace(abs_path, each_file_code_list):


    new_content=[]
    new_sorted_code_list=transform_line_no(each_file_code_list)
    group_new_sorted_code_list=[]
    prelineno=None
    for ind_group, (start_row, end_row, compli_code,simple_code,idiom,lineno) \
        in enumerate(new_sorted_code_list):
        if not prelineno:
            group_new_sorted_code_list.append([[start_row, end_row,compli_code,simple_code,idiom,lineno]])
            prelineno = start_row, end_row
            continue
        if start_row<=prelineno[1]:# if start_row <>>= group_new_sorted_code_list[-1][0][0] and end_row <= group_new_sorted_code_list[-1][0][1]:
            group_new_sorted_code_list[-1].append([start_row, end_row,compli_code,simple_code,idiom,lineno])
        else:
            group_new_sorted_code_list.append([[start_row, end_row,compli_code,simple_code,idiom,lineno]])
        if not prelineno:
            prelineno=start_row, end_row
        else:
            prelineno = min(start_row,prelineno[0]), max(end_row,prelineno[1])
    with open(abs_path) as f:
        lines=f.readlines()
        # print("lines: ",lines)
        # return None
        pre_lineno=None
        for ind_group,each_group_code in  enumerate(group_new_sorted_code_list):
            # print("1. each_group_code: ",each_group_code,len(each_group_code))
            bias=each_group_code[0][0]-1
            code_fragment_list = lines[each_group_code[0][0]-1:each_group_code[0][1]]
            # print("2. code_fragment_list: ",code_fragment_list)
            code_fragment_list_copy=copy.deepcopy(code_fragment_list)
            count_space = 0
            for e in code_fragment_list[0]:
                if e == " ":
                    count_space += 1
                else:
                    break

            for ind_each_code,each_code in enumerate(each_group_code):
                start_row, end_row, compli_code, simple_code, idiom, lineno = each_code
                if not pre_lineno and start_row-1 > 0:
                    new_content.extend(lines[len(new_content):start_row-1])
                    # print("add some extra*********new_content:\n",start_row,"".join(new_content),"*******")

                elif pre_lineno and pre_lineno[1]<start_row-1:
                    new_content.extend(lines[pre_lineno[1]:start_row - 1])
                    # print("add some extra*********new_content:\n",pre_lineno[1], start_row,"".join(new_content),"*******")

                if len(each_group_code)==1:
                    lineno_list = each_code[-1]
                    # compli_code=chain_compare_code[-4]
                    simp_code = each_code[-3]
                    code_fragment_list = replace_code(lineno_list,simp_code, code_fragment_list,bias)
                    new_content.extend(code_fragment_list)
                else:
                    # break
                    # if ind_each_code==0:
                    # print("count_space: ",count_space)
                    for ind_code_frag,e in enumerate(code_fragment_list):
                        if e!="\n":
                            code_fragment_list[ind_code_frag] = code_fragment_list[ind_code_frag][count_space:]
                    # print(">>>>>>>>>>>>>Idiom: ",ind_each_code,idiom,code_fragment_list,"\n","".join(code_fragment_list),count_space)

                    if idiom == 'List Comprehension':

                        # print(">>>List Comprehension code_fragment:\n","".join(code_fragment_list))
                        # print(">>>compli_code:\n","\n".join(compli_code))

                        code_pair_list = extract_compli_for_comprehension_only_one_stmt_improve.get_list_compreh_part_fragment("".join(code_fragment_list))

                        pass
                    elif idiom == 'Set Comprehension':
                        code_pair_list = extract_compli_for_comprehension_set_only_one_stmt.get_part_set_compreh("".join(code_fragment_list))
                        pass
                    elif idiom == 'Dict Comprehension':
                        code_pair_list = extract_compli_for_comprehension_dict_only_one_stmt_new.get_part_dict_compreh(
                            "".join(code_fragment_list))
                        pass
                    elif idiom == 'For Multi Targets':
                        # print(" Come For Multiple Targets")
                        code_pair_list = extract_compli_var_unpack_for_target_improve_new.get_for_multi_tar_part_fragment\
                            ("".join(code_fragment_list))
                    elif idiom == 'For Else':
                        # print("For Else code_fragment_list: \n",code_fragment_list,"".join(code_fragment_list))
                        code_pair_list = transform_for_else_compli_to_simple_improve_copy_result_csv.transform_part_for_else_code(
                            "".join(code_fragment_list))
                        # return jsonify(code_pair_list)

                        pass
                    elif idiom == 'Assign Multi Targets':
                        # print(" Come Assign Multiple Targets",code_fragment_list)
                        code_pair_list = extract_compli_multiple_assign_code_improve_complete_improve.transform_part_multiple_assign_code(
                            "".join(code_fragment_list))

                    elif idiom == "Truth Value Test":
                        k = 0
                        while 1:
                            try:
                                # print(" Come Truth Value Test", code_fragment_list)
                                tree = ast.parse("".join(code_fragment_list))
                                code_pair_list = extract_compli_truth_value_test_code_remove_is_isnot.get_part_truth_value_test_code("".join(code_fragment_list))
                                break
                            except:
                                k += 1
                                code_fragment_list.append(lines[each_group_code[0][1]][count_space:])
                        if k:
                            code_fragment_list=code_fragment_list[:-k]

                        # code_pair_list = extract_compli_multiple_assign_code_improve_complete_improve.transform_part_multiple_assign_code(
                        #     "".join(code_fragment_list))
                    elif idiom == "Chain Compare":
                        k = 0
                        while 1:
                            try:
                                tree = ast.parse("".join(code_fragment_list))
                                # print(" Come Chain Compare", code_fragment_list)
                                code_pair_list = transform_chained_comparison_compli_to_simple.get_part_chain_compare("".join(code_fragment_list))
                                break
                            except:
                                k += 1
                                code_fragment_list.append(lines[each_group_code[0][1]][count_space:])
                        if k:
                                code_fragment_list = code_fragment_list[:-k]
                            # code_pair_list = extract_compli_truth_value_test_code_remove_is_isnot.get_part_truth_value_test_code("".join(code_fragment_list))
                    elif idiom == "Call Star":
                        k=0
                        while 1:
                            try:
                                # print(" Come Call Star", code_fragment_list)
                                tree = ast.parse("".join(code_fragment_list))
                                code_pair_list = extract_compli_var_unpack_star_call_improve.get_info_from_code_fragment(
                                    "".join(code_fragment_list))
                                break
                            except:
                                k+=1
                                code_fragment_list.append(lines[each_group_code[0][1]][count_space:])
                        if k:
                                code_fragment_list = code_fragment_list[:-k]


                    #     new_lineno_list = code_pair_list[0][-1]
                    # # compli_code=chain_compare_code[-4]
                    #     simp_code = code_pair_list[0][-2]
                    else:
                        # print(" Come other ",idiom)
                        for ind_code_frag, e in enumerate(code_fragment_list):
                            if e != "\n":
                                code_fragment_list[ind_code_frag] = " " * count_space +code_fragment_list[ind_code_frag]
                        if ind_each_code == len(each_group_code) - 1:
                            new_content.extend(code_fragment_list)
                            # print(">>>*********new_content:\n", "".join(new_content))
                        continue
                    if not code_pair_list:
                        for ind_code_frag, e in enumerate(code_fragment_list):
                            if e != "\n":
                                code_fragment_list[ind_code_frag] = " " * count_space +code_fragment_list[ind_code_frag]
                        if ind_each_code == len(each_group_code) - 1:
                            new_content.extend(code_fragment_list)
                            # print(">>>*********new_content:\n", "".join(new_content))

                        continue
                    # print("code_pair_list: ",code_pair_list)
                    new_lineno_list = code_pair_list[0][-1]
                    # # compli_code=chain_compare_code[-4]
                    simp_code = code_pair_list[0][-2]
                    code_fragment_list = replace_code(new_lineno_list, simp_code, code_fragment_list)
                    # print(">>>new replace code_fragment_list: ", code_fragment_list)
                    # code_fragment_list[0] = " " * count_space + code_fragment_list[0]
                    code_fragment_list=''.join(code_fragment_list)
                    code_fragment_list=code_fragment_list.split("\n")

                    for ind_trans, e in enumerate(code_fragment_list):
                        if e != "":
                            code_fragment_list[ind_trans] = " " * count_space + code_fragment_list[ind_trans]
                        if ind_trans!=len(code_fragment_list)-1:
                                code_fragment_list[ind_trans]+='\n'

                    # print(">>>*********new new replace code_fragment_list:\n", ''.join(code_fragment_list))

                    if ind_each_code==len(each_group_code)-1:
                        new_content.extend(code_fragment_list)
                        # print(">>>*********new_content:\n", "".join(new_content))

                # print("each_time_code: ",new_content,start_row, end_row)
                # print(">>>>>each code\n","".join(new_content))
                if pre_lineno:
                    pre_lineno=max(start_row,pre_lineno[0]), max(end_row,pre_lineno[1])
                else:
                    pre_lineno=start_row, end_row
                if ind_each_code==len(each_group_code)-1:
                    new_content.extend(lines[pre_lineno[1]:])



    return "".join(new_content)

def main(args):
    # args = parser.parse_args()
    filepath = args.filepath
    idiom = args.idiom
    # outputpath = args.outputpath
    outputpath = args.output_codepair
    output_dir= args.outputdir
    flag_outputdir_mergeallcodepair=args.flag_merge_allcodepair
    flag_console_output=args.flag_console_output
    # outputpath = args.codepair_ouputpath

    format_code_list=[]
    a = []
    for i in range(1):
        a.append(i)
    # envname = args.envname
    if os.path.isdir(filepath):
        if os.path.exists(output_dir):
            shutil.rmtree(output_dir)
        for path,dir_list,file_list in os.walk(filepath):
            for file_name in file_list:
                abs_path=os.path.join(path,file_name)
                save_dir=path[len(filepath):]
                # print(">>>>>save_dir: ",save_dir)
                if abs_path.endswith(".py"):
                    print(f"************For File {abs_path.split('/')[-1]}, begin to find refactorable non-idiomatic code with Python idioms************")

                    code_frag=util.load_file_path(file_path=abs_path)
                    each_file_code_list,dict_code_pair=get_refactoring(idiom, code_frag,abs_path)
                    # print(code_pair_list)
                    if not flag_outputdir_mergeallcodepair:

                        for ind_save,each_code_pair in enumerate(each_file_code_list):
                            new_content=file_replace(abs_path,[each_code_pair])
                            # print(">>>>>output_dir: ", output_dir,save_dir)
                            util.save_file_path("".join([output_dir,save_dir,"/",file_name[:-3],'_',str(ind_save),'.py']),new_content)

                    else:
                        new_content=file_replace(abs_path,each_file_code_list)

                        util.save_file_path("".join([output_dir,save_dir,"/",file_name]),new_content)
                    print(f"************Result of {abs_path.split('/')[-1]} End************")

                    format_code_list.extend(each_file_code_list)
    else:
        file_name=filepath.split("/")[-1]
        print(
            f"************For File {filepath.split('/')[-1]}, begin to find refactorable non-idiomatic code with Python idioms************")
        code_frag = util.load_file_path(file_path=filepath)

        format_code_list,dict_code_pair=get_refactoring(idiom, code_frag,filepath)
        if not flag_outputdir_mergeallcodepair:
            if os.path.exists(output_dir):
                shutil.rmtree(output_dir)
            for ind_save, each_code_pair in enumerate(format_code_list):
                new_content = file_replace(filepath, [each_code_pair])

                util.save_file_path("".join([output_dir, "/", file_name[:-3], '_', str(ind_save), '.py']),
                                    new_content)


                # util.save_file_path()

        else:
            new_content = file_replace(filepath, format_code_list)
            if os.path.exists(output_dir):
                shutil.rmtree(output_dir)
            util.save_file_path("".join([output_dir, "/",file_name] ), new_content)

            # util.save_file_path()
        print(f"************Result of {filepath.split('/')[-1]} End************")

    print(f"************Result is saved in {outputpath}************", )
    util.save_json_file_path(outputpath, [e.__dict__ for e in format_code_list])
    # print(args)
    # print("main.py main() finished!")
if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='RefactoringIdioms')
    parser.add_argument('--envname', type=str,
                        help='envname', default="CartPole-v0")
    parser.add_argument('--filepath', type=str,
                        help='filepath', default=abs_path+"/file.py")
    # parser.add_argument('--filepath', type=str,
    #                     help='filepath', default=abs_path + "/")
    # parser.add_argument('--filepath', type=str,
    #                     help='filepath', default= "./")
    # parser.add_argument('--filepath', type=str,
    #                     help='filepath', default="/Users/zhangzejunzhangzejun/PycharmProjects/code")
    #
    parser.add_argument('--idiom', type=str,
                        help='idiom', default="All")
    parser.add_argument('--outputdir', type=str,
                        help='outputdir', default="./RefactoringIdiomsOutputdir/")
    # parser.add_argument('--outputdir', type=str,
    #                     help='outputdir', default="/Users/zhangzejunzhangzejun/PycharmProjects/code/RefactoringIdiomsOutputdir/")
    #
    # parser.add_argument('--flag_merge_allcodepair', type=str,
    #                     help='flag_merge_allcodepair', default=1)
    parser.add_argument('--flag_merge_allcodepair', type=str,
                        help='flag_merge_allcodepair', default=0)
    parser.add_argument('--output_codepair', type=str,
                        help='output_codepair', default="result.json")
    parser.add_argument('--flag_console_output', type=str,
                        help='flag_console_output', default=1)

    # parser.add_argument('--codepair_ouputpath', type=str,
    #                     help='codepair_ouputpath', default="result.json")
    # parser.add_argument('--outputdir', type=str,
    #                     help='outputdir', default="./RIdioms_dir/")

    args = parser.parse_args()
    print(args)
    main(args)
    filepath = args.filepath

    idiom = args.idiom
    # outputpath = args.codepair_ouputpath

    a=[]
    for i in range(1):
        a.append(i)
    # envname = args.envname
    # get_refactoring(idiom,filepath,outputpath)
    print(args)
    print("finished!")
    #    envname='MountainCar-v0'
    # env = gym.make(envname)
    # print_hi('PyCharm')

# See PyCharm help at https://www.jetbrains.com/help/pycharm/



# import RefactoringIdioms,argparse,os
# abs_path=os.path.abspath(os.path.dirname(__file__))
# # print("abs_path: ",abs_path)
# parser = argparse.ArgumentParser(description='RefactoringIdioms')
# parser.add_argument('--envname', type=str,
#                     help='envname', default="CartPole-v0")
# parser.add_argument('--filepath', type=str,
#                     help='filepath', default=abs_path+"/main.py")
# parser.add_argument('--idiom', type=str,
#                     help='idiom', default="All")
# parser.add_argument('--outputpath', type=str,
#                     help='outputpath', default="result.json")
# args = parser.parse_args()
# RefactoringIdioms.run_main(args)